@@ -2,6 +2,7 @@ use tokio::task::JoinHandle;
 use std::hash::Hash;
 use futures::Future;
 use crate::internal_cache::{CacheAction, InternalCacheStore, CacheMessage};
+use crate::backing::{CacheBacking, HashMapBacking};
 
 #[derive(Debug, Clone)]
 pub struct CacheLoadingError {
@@ -9,6 +10,12 @@ pub struct CacheLoadingError {
     // todo: nested errors
 }
 
+#[derive(Debug, Clone)]
+pub enum CacheEntry<V> {
+    Loaded(V),
+    Loading(tokio::sync::broadcast::Sender<Option<V>>),
+}
+
 #[derive(Debug)]
 pub enum CacheResult<V> {
     Found(V),
@@ -43,6 +50,7 @@ impl<
     /// # Examples
     ///
     /// ```
+    /// async fn example() {
     ///     use cache_loader_async::cache_api::LoadingCache;
     ///     use std::collections::HashMap;
     ///     let static_db: HashMap<String, u32> =
@@ -60,12 +68,20 @@ impl<
     ///     let result = cache.get("foo".to_owned()).await.unwrap();
     ///
     ///     assert_eq!(result, 32);
+    /// }
     /// ```
-    pub fn new<T, F>(loader: T) -> (LoadingCache<K, V, >, CacheHandle)
+    pub fn new<T, F>(loader: T) -> (LoadingCache<K, V>, CacheHandle)
         where F: Future<Output=Option<V>> + Sized + Send + 'static,
               T: Fn(K) -> F + Send + 'static {
+        LoadingCache::with_backing(HashMapBacking::new(), loader)
+    }
+
+    pub fn with_backing<T, F, B>(backing: B, loader: T) -> (LoadingCache<K, V>, CacheHandle)
+        where F: Future<Output=Option<V>> + Sized + Send + 'static,
+              T: Fn(K) -> F + Send + 'static,
+              B: CacheBacking<K, CacheEntry<V>> + Send + 'static {
         let (tx, rx) = tokio::sync::mpsc::channel(128);
-        let store = InternalCacheStore::new(tx.clone(), loader);
+        let store = InternalCacheStore::new(backing, tx.clone(), loader);
         let handle = store.run(rx);
         (LoadingCache {
             tx
@@ -106,6 +122,19 @@ impl<
         self.send_cache_action(CacheAction::Set(key, value)).await
     }
 
+    pub async fn get_if_present(&self, key: K) -> Result<Option<V>, CacheLoadingError> {
+        self.send_cache_action(CacheAction::GetIfPresent(key)).await
+    }
+
+    pub async fn exists(&self, key: K) -> Result<bool, CacheLoadingError> {
+        self.get_if_present(key).await
+            .map(|result| result.is_some())
+    }
+
+    pub async fn remove(&self, key: K) -> Result<Option<V>, CacheLoadingError> {
+        self.send_cache_action(CacheAction::Remove(key)).await
+    }
+
     /// Unstable, Undocumented & Inconsistent. Don't use that just yet
     pub async fn update<U>(&self, key: K, update_fn: U) -> Result<V, CacheLoadingError>
         where U: FnOnce(V) -> V + Send + 'static {
